package com.compomics.marc;

import com.compomics.util.io.export.styles.DefaultStyle;
import com.compomics.util.io.export.writers.ExcelWriter;
import com.compomics.util.math.BasicMathFunctions;
import com.compomics.util.math.roc.DataRoc;
import com.compomics.util.math.roc.DistributionRoc;
import com.compomics.util.math.statistics.ROC;
import com.compomics.util.math.statistics.distributions.NonSymmetricalNormalDistribution;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import org.apache.commons.math.MathException;
import org.apache.commons.math.util.MathUtils;

/**
 *
 * @author Marc
 */
public class RocScript {

    private static final String separator = "\t";

    public static void main(String[] args) {
        RocScript m = new RocScript();

        try {
            m.jillRoc();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void jillRoc() throws FileNotFoundException, IOException, MathException {

        // Input
        File workingFolder = new File("D:\\projects\\jill roc\\");
        File patientMappingFile = new File(workingFolder, "patient mapping.txt");

        HashMap<String, String> patientNameToCategory = new HashMap<String, String>();
        HashMap<String, String> patientKeyToCategory = new HashMap<String, String>();
        HashMap<String, String> patientNameToKey = new HashMap<String, String>();
        loadPatientMapping(patientMappingFile, patientNameToCategory, patientKeyToCategory, patientNameToKey);

        File inputUnaffected = new File(workingFolder, "blood unaffected.txt");
        File inputUncertain = new File(workingFolder, "blood affect uncertain.txt");
        File inputAffected = new File(workingFolder, "blood affected.txt");
        HashMap<String, Integer> proteinAccessionToCategory = loadProteinMapping(inputUnaffected, inputUncertain, inputAffected);

        File concentrationRawFile = new File(workingFolder, "concentration raw.txt");
        File concentrationNormalizedFile = new File(workingFolder, "concentration normalized.txt");
        File volumeRawFile = new File(workingFolder, "volume raw.txt");
        File volumeNormalizedFile = new File(workingFolder, "volume normalized.txt");

        HashMap<String, HashMap<String, Double>> controlInputConcentrationRaw = new HashMap<String, HashMap<String, Double>>(),
                ondInputConcentrationRaw = new HashMap<String, HashMap<String, Double>>(),
                sasInputConcentrationRaw = new HashMap<String, HashMap<String, Double>>(),
                rrmsConcentrationRaw = new HashMap<String, HashMap<String, Double>>(),
                controlInputVolumeRaw = new HashMap<String, HashMap<String, Double>>(),
                ondInputVolumeRaw = new HashMap<String, HashMap<String, Double>>(),
                sasInputVolumeRaw = new HashMap<String, HashMap<String, Double>>(),
                rrmsVolumeRaw = new HashMap<String, HashMap<String, Double>>(),
                controlInputConcentrationNormalized = new HashMap<String, HashMap<String, Double>>(),
                ondInputConcentrationNormalized = new HashMap<String, HashMap<String, Double>>(),
                sasInputConcentrationNormalized = new HashMap<String, HashMap<String, Double>>(),
                rrmsConcentrationNormalized = new HashMap<String, HashMap<String, Double>>(),
                controlInputVolumeNormalized = new HashMap<String, HashMap<String, Double>>(),
                ondInputVolumeNormalized = new HashMap<String, HashMap<String, Double>>(),
                sasInputVolumeNormalized = new HashMap<String, HashMap<String, Double>>(),
                rrmsVolumeNormalized = new HashMap<String, HashMap<String, Double>>();

        jillLoadRoc(concentrationRawFile, patientNameToCategory, patientNameToKey, controlInputConcentrationRaw, sasInputConcentrationRaw, ondInputConcentrationRaw, rrmsConcentrationRaw);
        jillLoadRoc(concentrationNormalizedFile, patientNameToCategory, patientNameToKey, controlInputConcentrationNormalized, sasInputConcentrationNormalized, ondInputConcentrationNormalized, rrmsConcentrationNormalized);
        jillLoadRoc(volumeRawFile, patientKeyToCategory, null, controlInputVolumeRaw, sasInputVolumeRaw, ondInputVolumeRaw, rrmsVolumeRaw);
        jillLoadRoc(volumeNormalizedFile, patientKeyToCategory, null, controlInputVolumeNormalized, sasInputVolumeNormalized, ondInputVolumeNormalized, rrmsVolumeNormalized);

        File output = new File(workingFolder, "Volume variability.txt");
        outputForNormalization(output, proteinAccessionToCategory, sasInputVolumeRaw, ondInputVolumeRaw, rrmsVolumeRaw);
        output = new File(workingFolder, "Concentration variability.txt");
        outputForNormalization(output, proteinAccessionToCategory, sasInputConcentrationRaw, ondInputConcentrationRaw, rrmsConcentrationRaw);
        
        output = new File(workingFolder, "Raw Matching.txt");
        outputForMatching(output, sasInputConcentrationRaw, sasInputVolumeRaw, ondInputConcentrationRaw, ondInputVolumeRaw, rrmsConcentrationRaw, rrmsVolumeRaw);
        output = new File(workingFolder, "Normalized Matching.txt");
        outputForMatching(output, sasInputConcentrationNormalized, sasInputVolumeNormalized, ondInputConcentrationNormalized, ondInputVolumeNormalized, rrmsConcentrationNormalized, rrmsVolumeNormalized);
        
        int end = 1;
    }
    
    private void outputForMatching(File outputFile, HashMap<String, HashMap<String, Double>> sasConcentration, HashMap<String, HashMap<String, Double>> sasVolume, 
            HashMap<String, HashMap<String, Double>> ondConcentration, HashMap<String, HashMap<String, Double>> ondVolume, 
            HashMap<String, HashMap<String, Double>> rrmsConcentration, HashMap<String, HashMap<String, Double>> rrmsVolume) throws IOException {
        
        BufferedWriter bw = new BufferedWriter(new FileWriter(outputFile));
        try {
            bw.write("group" + separator + "accession" + separator + "patient" + separator + "mean" + separator + "sd");
            bw.newLine();
            outputGroupValuesForMatching(bw, "SAS", sasConcentration, sasVolume);
            outputGroupValuesForMatching(bw, "OND", ondConcentration, ondVolume);
            outputGroupValuesForMatching(bw, "RRMS", rrmsConcentration, rrmsVolume);
        } finally {
            bw.close();
        }
    }
    
    private void outputGroupValuesForMatching(BufferedWriter bw, String group, HashMap<String, HashMap<String, Double>> concentration, HashMap<String, HashMap<String, Double>> volume) throws IOException {
        for (String accession : concentration.keySet()) {
            HashMap<String, Double> concentrationValues = concentration.get(accession);
            HashMap<String, Double> volumeValues = volume.get(accession);
            if (volumeValues != null) {
                for (String patient : concentrationValues.keySet()) {
                    Double concentrationValue = concentrationValues.get(patient);
                    Double volumeValue = volumeValues.get(patient);
                    if (volumeValue != null) {
                        double mean = (concentrationValue + volumeValue) / 2;
                        double std = concentrationValue - volumeValue;
                        bw.write(group + separator + accession + separator + patient + separator + mean + separator + std);
                        bw.newLine();
                    }
                }
            }
        }
    }

    private void outputForNormalization(File outputFile, HashMap<String, Integer> proteinAccessionToCategory, 
            HashMap<String, HashMap<String, Double>> sasInput, HashMap<String, HashMap<String, Double>> ondInput, HashMap<String, HashMap<String, Double>> rrmsInput) throws IOException {

        BufferedWriter bw = new BufferedWriter(new FileWriter(outputFile));
        try {
            int nColumns = 0;
            for (HashMap<String, Double> intensities : sasInput.values()) {
                int length = intensities.size();
                if (length > nColumns) {
                    nColumns = length;
                }
            }
            bw.write("Category");
            for (int i = 1 ; i <= nColumns ; i++) {
                bw.write(separator + i);
            }
            bw.newLine();
            
            writeStds(bw, proteinAccessionToCategory, sasInput);
            writeStds(bw, proteinAccessionToCategory, ondInput);
            writeStds(bw, proteinAccessionToCategory, rrmsInput);
            
        } finally {
            bw.close();
        }
    }
    
    private void writeStds(BufferedWriter bw, HashMap<String, Integer> proteinAccessionToCategory, HashMap<String, HashMap<String, Double>> input) throws IOException {
            
            for (String accession : input.keySet()) {
                ArrayList<Double> intensities = new ArrayList<Double>(input.get(accession).values());
                if (intensities.size() > 3) {
                    Integer category = proteinAccessionToCategory.get(accession);
                    if (category == null) {
                        category = 4;
                    }
                    double median = BasicMathFunctions.median(intensities);
                    double totalIntensity = BasicMathFunctions.sum(intensities);
                    if (median > 0) {
                        ArrayList<Double> normalizedIntensities = new ArrayList<Double>();
                        for (double intensity : intensities) {
                            double normalizedIntensity = intensity / median;
                            normalizedIntensities.add(normalizedIntensity);
                        }
                        bw.write(category + "");
                        double std = BasicMathFunctions.std(intensities) / median;
                        bw.write(separator + std + separator + totalIntensity);
                        bw.newLine();
                    }
                }
            }
    }
    
    private void writeNormalizedInputForPCA(BufferedWriter bw, HashMap<String, Integer> proteinAccessionToCategory, HashMap<String, ArrayList<Double>> input) throws IOException {
            
            for (String accession : input.keySet()) {
                ArrayList<Double> intensities = input.get(accession);
                if (intensities.size() > 3) {
                    Integer category = proteinAccessionToCategory.get(accession);
                    if (category == null) {
                        category = 4;
                    }
                    double median = BasicMathFunctions.median(intensities);
                    if (median > 0) {
                        ArrayList<Double> normalizedIntensities = new ArrayList<Double>();
                        for (double intensity : intensities) {
                            double normalizedIntensity = intensity / median;
                            normalizedIntensities.add(normalizedIntensity);
                        }
                        bw.write(category + "");
                        for (double intensity : normalizedIntensities) {
                            bw.write(separator + intensity);
                        }
                        bw.newLine();
                    }
                }
            }
    }
    /**
     * 0 -> unaffected 
     * 1 -> uncertain 
     * 2 -> affected
     * null -> other
     *
     * @param inputUnaffected
     * @param inputUncertain
     * @param inputAffected
     * @return
     */
    private HashMap<String, Integer> loadProteinMapping(File inputUnaffected, File inputUncertain, File inputAffected) throws IOException {
        HashMap<String, Integer> result = new HashMap<String, Integer>();

        BufferedReader br = new BufferedReader(new FileReader(inputUnaffected));
        try {
            String line = br.readLine();
            while ((line = br.readLine()) != null) {
                String name = line.trim();
                result.put(name, 0);
            }
        } finally {
            br.close();
        }

        br = new BufferedReader(new FileReader(inputUncertain));
        try {
            String line = br.readLine();
            while ((line = br.readLine()) != null) {
                String name = line.trim();
                result.put(name, 1);
            }
        } finally {
            br.close();
        }

        br = new BufferedReader(new FileReader(inputAffected));
        try {
            String line = br.readLine();
            while ((line = br.readLine()) != null) {
                String name = line.trim();
                result.put(name, 2);
            }
        } finally {
            br.close();
        }

        return result;
    }

    private void loadPatientMapping(File inputFile, HashMap<String, String> nameToCategory, HashMap<String, String> keyToCategory, HashMap<String, String> nameToKey) throws IOException {

        BufferedReader br = new BufferedReader(new FileReader(inputFile));
        try {
            String line = br.readLine();
            while ((line = br.readLine()) != null) {
                String[] content = line.split("\t");
                String name = content[0];
                String key = content[1];
                String category = content[2];
                nameToCategory.put(name, category);
                keyToCategory.put(key, category);
                nameToKey.put(name, key);
            }
        } finally {
            br.close();
        }
    }

    private void jillLoadRoc(File inputFile, HashMap<String, String> categoryMapping, HashMap<String, String> nameToKey, HashMap<String, HashMap<String, Double>> controlMap, HashMap<String, HashMap<String, Double>> sasMap, HashMap<String, HashMap<String, Double>> ondMap, HashMap<String, HashMap<String, Double>> rrmsMap) throws FileNotFoundException, IOException {

        BufferedReader br = new BufferedReader(new FileReader(inputFile));
        try {

            String line = br.readLine();
            ArrayList<String> names = new ArrayList<String>();
            String[] content = line.split("\t");
            for (int i = 5; i < 106; i++) {
                names.add(content[i]);
            }

            while ((line = br.readLine()) != null) {
                content = line.split("\t");
                String accession = content[0];
                for (int i = 5; i < 106; i++) {
                    String name = names.get(i - 5);
                    String category = categoryMapping.get(name);
                    if (category == null) {
                        throw new IllegalArgumentException("Name not recognized " + name + ".");
                    }
                    String key = name;
                    if (nameToKey != null) {
                        key = nameToKey.get(name);
                    }
                    String component = "";
                    if (content.length > i) {
                        component = content[i].trim();
                    }
                    if (!component.equals("")) {
                        Double value;
                        try {
                            value = new Double(content[i]);
                        } catch (Exception e) {
                            throw new IllegalArgumentException("Failed to convert " + content[i] + ", " + i + " of accession " + accession);
                        }
                        if (category.equals("SAS")) {
                            HashMap<String, Double> sasIntensities = sasMap.get(accession);
                            if (sasIntensities == null) {
                                sasIntensities = new HashMap<String, Double>();
                                sasMap.put(accession, sasIntensities);
                            }
                            sasIntensities.put(key, value);

                            HashMap<String, Double> controlIntensities = controlMap.get(accession);
                            if (controlIntensities == null) {
                                controlIntensities = new HashMap<String, Double>();
                                controlMap.put(accession, controlIntensities);
                            }
                            controlIntensities.put(key, value);
                        } else if (category.equals("OND")) {
                            HashMap<String, Double> ondIntensities = ondMap.get(accession);
                            if (ondIntensities == null) {
                                ondIntensities = new HashMap<String, Double>();
                                ondMap.put(accession, ondIntensities);
                            }
                            ondIntensities.put(key, value);

                            HashMap<String, Double> controlIntensities = controlMap.get(accession);
                            if (controlIntensities == null) {
                                controlIntensities = new HashMap<String, Double>();
                                controlMap.put(accession, controlIntensities);
                            }
                            controlIntensities.put(key, value);
                        } else if (category.equals("RRMS")) {
                            HashMap<String, Double> rrmsIntensities = rrmsMap.get(accession);
                            if (rrmsIntensities == null) {
                                rrmsIntensities = new HashMap<String, Double>();
                                rrmsMap.put(accession, rrmsIntensities);
                            }
                            rrmsIntensities.put(key, value);
                        } else {
                            throw new IllegalArgumentException("Name not recognized " + name + ".");
                        }
                    }
                }
            }

        } finally {
            br.close();
        }
    }

    public void yngvildRoc() throws FileNotFoundException, IOException, MathException {

        // Input
        File inFile = new File("D:\\projects\\biomarkers Yngvild\\input_roc.txt");

        HashMap<String, ArrayList<Double>> controlInput = new HashMap<String, ArrayList<Double>>(),
                healthyMutInput = new HashMap<String, ArrayList<Double>>(),
                diabetesInput = new HashMap<String, ArrayList<Double>>();
        HashMap<String, NonSymmetricalNormalDistribution> controlDistributions = new HashMap<String, NonSymmetricalNormalDistribution>(),
                healthyMutDistributions = new HashMap<String, NonSymmetricalNormalDistribution>(),
                diabetesMutationDistributions = new HashMap<String, NonSymmetricalNormalDistribution>();
        HashMap<String, ROC> mutationDistributionRocs = new HashMap<String, ROC>(),
                mutationDataRocs = new HashMap<String, ROC>(),
                diabetesDistributionRocs = new HashMap<String, ROC>(),
                diabetesDataRocs = new HashMap<String, ROC>();
        ArrayList<String> transitions = new ArrayList<String>();
        HashMap<String, Double> lods = new HashMap<String, Double>();
        HashMap<String, Integer> ns = new HashMap<String, Integer>();
        HashMap<String, Integer> nsNotNull = new HashMap<String, Integer>();

        BufferedReader br = new BufferedReader(new FileReader(inFile));
        try {
            ArrayList<StringBuilder> transitionsTemp = new ArrayList<StringBuilder>();
            String line = br.readLine();
            String[] split = line.split(separator);
            for (int cpt = 1; cpt < split.length; cpt++) {
                transitionsTemp.add(new StringBuilder(split[cpt]));
            }
            line = br.readLine();
            split = line.split(separator);
            for (int cpt = 1; cpt < split.length; cpt++) {
                String peptide = split[cpt];
                peptide = peptide.trim();
                if (peptide.length() > 0) {
                    StringBuilder transition = transitionsTemp.get(cpt - 1);
                    transition.append(" ").append(peptide);
                }
            }
            line = br.readLine();
            split = line.split(separator);
            for (int cpt = 1; cpt < split.length; cpt++) {
                String ion = split[cpt];
                ion = ion.trim();
                if (ion.length() > 0) {
                    StringBuilder transition = transitionsTemp.get(cpt - 1);
                    transition.append(" ").append(ion);
                }
            }
            for (StringBuilder transition : transitionsTemp) {
                transitions.add(transition.toString());
            }

            while ((line = br.readLine()) != null) {
                line = line.trim();
                if (!line.equals("")) {
                    split = line.split(separator);
                    Integer patientClass = new Integer(split[0]);
                    for (int cpt = 1; cpt < split.length; cpt++) {
                        String transition = transitions.get(cpt - 1);
                        ArrayList<Double> data;
                        if (patientClass == 0) {
                            data = controlInput.get(transition);
                            if (data == null) {
                                data = new ArrayList<Double>();
                                controlInput.put(transition, data);
                            }
                        } else if (patientClass == 1) {
                            data = healthyMutInput.get(transition);
                            if (data == null) {
                                data = new ArrayList<Double>();
                                healthyMutInput.put(transition, data);
                            }
                        } else {
                            data = diabetesInput.get(transition);
                            if (data == null) {
                                data = new ArrayList<Double>();
                                diabetesInput.put(transition, data);
                            }
                        }
                        try {
                            Double value = new Double(split[cpt]);
                            data.add(value);

                            // LOD
                            if (value > 0) {
                                Double lod = lods.get(transition);
                                if (lod == null || lod > value) {
                                    lods.put(transition, value);
                                }
                            }

                            // n
                            Integer n = ns.get(transition);
                            if (n == null) {
                                ns.put(transition, 1);
                            } else {
                                ns.put(transition, n + 1);
                            }

                            // n>0
                            if (value > 0) {
                                n = nsNotNull.get(transition);
                                if (n == null) {
                                    nsNotNull.put(transition, 1);
                                } else {
                                    nsNotNull.put(transition, n + 1);
                                }
                            }

                        } catch (Exception e) {
                            // not a number
                        }
                    }
                }
            }
        } finally {
            br.close();
        }

        // Distributions & ROCS
        for (String transition : transitions) {
            ArrayList<Double> controlValues = controlInput.get(transition);
            NonSymmetricalNormalDistribution controlDistribution = null,
                    healthyMutDistribution = null,
                    diabetesDistribution = null;
            if (controlValues != null && !controlValues.isEmpty()) {
                controlDistribution = NonSymmetricalNormalDistribution.getRobustNonSymmetricalNormalDistribution(controlValues);
                controlDistributions.put(transition, controlDistribution);
            }
            ArrayList<Double> healthyMutValues = healthyMutInput.get(transition);
            if (healthyMutValues != null && !healthyMutValues.isEmpty()) {
                healthyMutDistribution = NonSymmetricalNormalDistribution.getRobustNonSymmetricalNormalDistribution(healthyMutValues);
                healthyMutDistributions.put(transition, healthyMutDistribution);
            }
            ArrayList<Double> diabetesValues = diabetesInput.get(transition);
            if (diabetesValues != null && !diabetesValues.isEmpty()) {
                diabetesDistribution = NonSymmetricalNormalDistribution.getRobustNonSymmetricalNormalDistribution(diabetesValues);
                diabetesMutationDistributions.put(transition, diabetesDistribution);
            }

            if (controlDistribution != null && healthyMutDistribution != null) {
                DistributionRoc distributionRoc = new DistributionRoc(controlDistribution, healthyMutDistribution);
                mutationDistributionRocs.put(transition, distributionRoc);
                DataRoc dataRoc = new DataRoc(controlValues, healthyMutValues, DataRoc.RocInterpolation.minimum);
                mutationDataRocs.put(transition, dataRoc);
            }
            if (healthyMutDistribution != null && diabetesDistribution != null) {
                DistributionRoc distributionRoc = new DistributionRoc(healthyMutDistribution, diabetesDistribution);
                diabetesDistributionRocs.put(transition, distributionRoc);
                DataRoc dataRoc = new DataRoc(healthyMutValues, diabetesValues, DataRoc.RocInterpolation.minimum);
                diabetesDataRocs.put(transition, dataRoc);
            }
        }

        // output
        File outFile = new File("D:\\projects\\biomarkers Yngvild\\30.09.14_rocs.xls");
        ExcelWriter ew = new ExcelWriter(outFile);
        DefaultStyle style = DefaultStyle.getStyle(ew);
        ew.setWorkbookStyle(style);

        ew.startNewSection("Summary");

        ew.writeHeaderText("Transition");
        ew.addSeparator();
        ew.writeHeaderText("N");
        ew.addSeparator();
        ew.writeHeaderText("N>0");
        ew.addSeparator();
        ew.writeHeaderText("LOD");
        ew.addSeparator();
        ew.writeHeaderText("Healthy Mut Vs Control - AUC (Empirical)");
        ew.addSeparator();
        ew.writeHeaderText("Healthy Mut Vs Control - AUC (Statistical)");
        ew.addSeparator();
        ew.writeHeaderText("Diabetes Vs Healthy Mut - AUC (Empirical)");
        ew.addSeparator();
        ew.writeHeaderText("Diabetes Vs Healthy Mut - AUC (Statistical)");
        ew.newLine();
        for (String transition : transitions) {
            ew.writeHeaderText(transition);
            ew.addSeparator();
            ew.write(ns.get(transition) + "");
            ew.addSeparator();
            ew.write(nsNotNull.get(transition) + "");
            ew.addSeparator();
            ew.write(lods.get(transition) + "");
            ew.addSeparator();
            ROC roc = mutationDataRocs.get(transition);
            if (roc != null) {
                ew.write(roc.getAuc() + "");
            }
            ew.addSeparator();
            roc = mutationDistributionRocs.get(transition);
            if (roc != null) {
                ew.write(roc.getAuc() + "");
            }
            ew.addSeparator();
            roc = diabetesDataRocs.get(transition);
            if (roc != null) {
                ew.write(roc.getAuc() + "");
            }
            ew.addSeparator();
            roc = diabetesDistributionRocs.get(transition);
            if (roc != null) {
                ew.write(roc.getAuc() + "");
            }
            ew.newLine();
        }

        ew.startNewSection("Mutation Healthy Vs Control");

        for (String transition : transitions) {
            ew.writeHeaderText(transition);
            ew.addSeparator();
            ew.addSeparator();
            ew.addSeparator();
            ew.addSeparator();
        }
        ew.newLine();

        // ROC titles
        for (String transition : transitions) {
            ew.writeHeaderText("ROC (Empirical)");
            ew.addSeparator();
            ew.addSeparator();
            ew.writeHeaderText("Statistical ROC (Statistical)");
            ew.addSeparator();
            ew.addSeparator();
        }
        ew.newLine();

        // AUC
        for (String transition : transitions) {
            ew.write("AUC");
            ew.addSeparator();
            ROC roc = mutationDataRocs.get(transition);
            if (roc != null) {
                ew.write(roc.getAuc() + "");
            }
            ew.addSeparator();
            ew.write("AUC");
            ew.addSeparator();
            roc = mutationDistributionRocs.get(transition);
            if (roc != null) {
                ew.write(roc.getAuc() + "");
            }
            ew.addSeparator();
        }
        ew.newLine();

        // ROC titles
        for (String transition : transitions) {
            ew.write("1-specificity");
            ew.addSeparator();
            ew.write("sensitivity");
            ew.addSeparator();
            ew.write("1-specificity");
            ew.addSeparator();
            ew.write("sensitivity");
            ew.addSeparator();
        }
        ew.newLine();

        // AUC
        int cpt = 0;
        boolean allDone = false;
        while (!allDone) {
            allDone = true;
            for (String transition : transitions) {
                ROC roc = mutationDataRocs.get(transition);
                if (roc != null) {
                    double[][] xy = roc.getxYValues();
                    if (cpt < xy.length) {
                        allDone = false;
                        ew.write(xy[cpt][0] + "");
                        ew.addSeparator();
                        ew.write(xy[cpt][1] + "");
                        ew.addSeparator();
                    } else {
                        ew.addSeparator();
                        ew.addSeparator();
                    }
                } else {
                    ew.addSeparator();
                    ew.addSeparator();
                }
                roc = mutationDistributionRocs.get(transition);
                if (roc != null) {
                    double[][] xy = roc.getxYValues();
                    if (cpt < xy.length) {
                        allDone = false;
                        ew.write(xy[cpt][0] + "");
                        ew.addSeparator();
                        ew.write(xy[cpt][1] + "");
                        ew.addSeparator();
                    } else {
                        ew.addSeparator();
                        ew.addSeparator();
                    }
                } else {
                    ew.addSeparator();
                    ew.addSeparator();
                }
            }
            ew.newLine();
            cpt++;
        }
        ew.newLine();

        // Comparison
        ew.startNewSection("Diabetes Vs Mutation Healthy");

        for (String transition : transitions) {
            ew.writeHeaderText(transition);
            ew.addSeparator();
            ew.addSeparator();
            ew.addSeparator();
            ew.addSeparator();
        }
        ew.newLine();

        // ROC titles
        for (String transition : transitions) {
            ew.writeHeaderText("Empirical ROC");
            ew.addSeparator();
            ew.addSeparator();
            ew.writeHeaderText("Statistical ROC");
            ew.addSeparator();
            ew.addSeparator();
        }
        ew.newLine();

        // AUC
        for (String transition : transitions) {
            ew.write("AUC");
            ew.addSeparator();
            ROC roc = diabetesDataRocs.get(transition);
            if (roc != null) {
                ew.write(roc.getAuc() + "");
            }
            ew.addSeparator();
            ew.write("AUC");
            ew.addSeparator();
            roc = diabetesDistributionRocs.get(transition);
            if (roc != null) {
                ew.write(roc.getAuc() + "");
            }
            ew.addSeparator();
        }
        ew.newLine();

        // ROC titles
        for (String transition : transitions) {
            ew.write("1-specificity");
            ew.addSeparator();
            ew.write("sensitivity");
            ew.addSeparator();
            ew.write("1-specificity");
            ew.addSeparator();
            ew.write("sensitivity");
            ew.addSeparator();
        }
        ew.newLine();

        // AUC
        cpt = 0;
        allDone = false;
        while (!allDone) {
            allDone = true;
            for (String transition : transitions) {
                ROC roc = diabetesDataRocs.get(transition);
                if (roc != null) {
                    double[][] xy = roc.getxYValues();
                    if (cpt < xy.length) {
                        allDone = false;
                        ew.write(xy[cpt][0] + "");
                        ew.addSeparator();
                        ew.write(xy[cpt][1] + "");
                        ew.addSeparator();
                    } else {
                        ew.addSeparator();
                        ew.addSeparator();
                    }
                } else {
                    ew.addSeparator();
                    ew.addSeparator();
                }
                roc = diabetesDistributionRocs.get(transition);
                if (roc != null) {
                    double[][] xy = roc.getxYValues();
                    if (cpt < xy.length) {
                        allDone = false;
                        ew.write(xy[cpt][0] + "");
                        ew.addSeparator();
                        ew.write(xy[cpt][1] + "");
                        ew.addSeparator();
                    } else {
                        ew.addSeparator();
                        ew.addSeparator();
                    }
                } else {
                    ew.addSeparator();
                    ew.addSeparator();
                }
            }
            ew.newLine();
            cpt++;
        }

        ew.close();

    }

    public void patientControlRoc() throws FileNotFoundException, IOException, MathException {

        // Input
        File inFile = new File("D:\\projects\\biomarkers Yngvild\\input_roc.txt");

        HashMap<String, ArrayList<Double>> controlInput = new HashMap<String, ArrayList<Double>>(),
                msInput = new HashMap<String, ArrayList<Double>>();
        HashMap<String, NonSymmetricalNormalDistribution> controlDistributions = new HashMap<String, NonSymmetricalNormalDistribution>(),
                msDistributions = new HashMap<String, NonSymmetricalNormalDistribution>();
        HashMap<String, ROC> rocs = new HashMap<String, ROC>();
        ArrayList<String> proteins = new ArrayList<String>();

        BufferedReader br = new BufferedReader(new FileReader(inFile));
        try {
            String line = br.readLine();
            while ((line = br.readLine()) != null) {
                String[] split = line.split(separator);
                String accession = split[0];
                proteins.add(accession);
                ArrayList<Double> input = new ArrayList<Double>();
                for (int i = 5; i <= 68; i++) {
                    String intensity = split[i];
                    if (!intensity.equals("")) {
                        input.add(new Double(intensity));
                    }
                }
                controlInput.put(accession, input);
                NonSymmetricalNormalDistribution controlDistribution = NonSymmetricalNormalDistribution.getRobustNonSymmetricalNormalDistribution(input);
                controlDistributions.put(accession, controlDistribution);
                input = new ArrayList<Double>();
                for (int i = 69; i <= 105; i++) {
                    String intensity = split[i];
                    if (!intensity.equals("")) {
                        input.add(new Double(intensity));
                    }
                }
                msInput.put(accession, input);
                NonSymmetricalNormalDistribution msDistribution = NonSymmetricalNormalDistribution.getRobustNonSymmetricalNormalDistribution(input);
                msDistributions.put(accession, msDistribution);

                DistributionRoc roc = new DistributionRoc(controlDistribution, msDistribution);
                rocs.put(accession, roc);
            }
        } finally {
            br.close();
        }

        // output
        File outFile = new File("D:\\projects\\jill roc\\rocs.txt");
        BufferedWriter bw = new BufferedWriter(new FileWriter(outFile));
        for (String accession : proteins) {
            bw.write(separator + accession);
        }
        bw.newLine();

        try {
            for (double x = 0; x <= 1; x += 0.01) {
                bw.write(x + "");
                for (String accession : proteins) {
                    ROC roc = rocs.get(accession);
                    double y = roc.getValueAt(x);
                    bw.write(separator + y);
                }
                bw.newLine();
            }
        } finally {
            bw.close();
        }

        // inverted output
        outFile = new File("D:\\projects\\jill roc\\inverted_rocs.txt");
        bw = new BufferedWriter(new FileWriter(outFile));
        for (String accession : proteins) {
            bw.write(separator + accession);
        }
        bw.newLine();

        try {
            for (double x = 0; x <= 1; x += 0.01) {
                bw.write(x + "");
                for (String accession : proteins) {
                    ROC roc = rocs.get(accession);
                    double y = roc.getSpecificityAt(x);
                    bw.write(separator + y);
                }
                bw.newLine();
            }
        } finally {
            bw.close();
        }
    }

}
